﻿using static Android.Text.TextUtils;

using System;
using System.Linq;
using System.Collections.Generic;
using MvvmCross.Platforms.Android.Views;
using MvvmCross.Platforms.Android.Binding.BindingContext;
using MvvmCross.Droid.Support.V7.RecyclerView;
using DDP.Common.Mvx.Droid.Adapters.Recycler;
using DDP.Common.Extensions;
using DDP.Common.Droid.Listeners;
using DDP.Common.Droid.Extensions;
using Android.Widget;
using Android.Views;
using Android.Util;
using Android.Support.V7.Widget;
using Android.Runtime;
using Android.Graphics;
using Android.Content;
using ILive.Core.Models.Interfaces;
using DDP.Common.Mvx.Collections.Interfaces;

namespace ILive.UIControls.Droid.QuickActionsView
{
    [Register("ilive.uicontrols.droid.QuickActionsView")]
    public class QuickActionsView : LinearLayout
    {
        //      #fields         ---     ---     ---

        const float CategoryTextSizeInSp = 17.0f;
        const float DefaultItemTitleTextSizeInSp = 16.0f;
        const float CategoryTitleLeftMarginInDp = 13.0f;
        const float NeighboringItemsSideMarginInDp = 6.0f;
        const string SelectedCategoryFontFamily = "sans-serif-medium";

        LinearLayout _categoriesContainer;
        MvxRecyclerView _itemsRecyclerView;
        BaseHeaderFooterMvxRecyclerAdapter _adapter;
        LinearLayoutManager _layoutManager;

        bool _simpleMode;
        Color _categoryTitleColor;
        Color _categorySelectedTitleColor;
        Typeface _categoryTitleTypeface;
        Typeface _categorySelectedTitleTypeface;

        int _itemBackgroundResource;
        int _itemSelectedBackgroundResource;
        float _itemTitleTextSize;
        Color _itemTitleColor;
        Color _itemSelectedTitleColor;
        Color _itemSubTitleColor;
        Color _itemSelectedSubTitleColor;
        int _itemIconResource;
        int _itemSelectedIconResource;

        IEnumerable<IMvxObservableGroup<string, IQuickActionSectionItem>> _items;
        IQuickActionSectionItem _selectedItem;
        int _selectedCategoryIndex = -1;
        bool _isScrollingByUser;


        //      #constructors   ---     ---     ---

        public QuickActionsView(Context context)
            : base(context)
        {
            InitView();
        }


        public QuickActionsView(Context context, IAttributeSet attrs)
            : base(context, attrs)
        {
            InitAttrs(context, attrs);
            InitView();
        }


        public QuickActionsView(Context context, IAttributeSet attrs, int defStyleAttr)
            : base(context, attrs, defStyleAttr)
        {
            InitAttrs(context, attrs);
            InitView();
        }


        public QuickActionsView(Context context, IAttributeSet attrs, int defStyleAttr, int defStyleRes)
            : base(context, attrs, defStyleAttr, defStyleRes)
        {
            InitAttrs(context, attrs);
            InitView();
        }


        protected QuickActionsView(IntPtr javaReference, JniHandleOwnership transfer)
            : base(javaReference, transfer)
        {
            // nothing here...
        }


        //      #properties     ---     ---     ---

        public IEnumerable<IMvxObservableGroup<string, IQuickActionSectionItem>> Items
        {
            get => _items;
            set
            {
                _items = value;
                InitializeItems();
            }
        }


        public IQuickActionSectionItem SelectedItem
        {
            get => _selectedItem;
            set
            {
                _selectedItem = value;
                SelectItem();
            }
        }


        //      #methods --     ---     ---

        void InitAttrs(Context context, IAttributeSet attrs)
        {
            var values = context.ObtainStyledAttributes(attrs, Resource.Styleable.QuickActionsView);

            try
            {
                _simpleMode = values.GetBoolean(Resource.Styleable.QuickActionsView_simpleMode, defValue: false);
                _categoryTitleColor = values.GetColor(Resource.Styleable.QuickActionsView_categoryTitleColor, Resource.Color.Gray);
                _categorySelectedTitleColor = values.GetColor(Resource.Styleable.QuickActionsView_categorySelectedTitleColor, Android.Resource.Color.Black);
                _itemBackgroundResource = values.GetResourceId(Resource.Styleable.QuickActionsView_itemBackgroundResource, Resource.Drawable.WhiteRounded13xCornersBackground);
                _itemSelectedBackgroundResource = values.GetResourceId(Resource.Styleable.QuickActionsView_itemSelectedBackgroundResource, 0);
                _itemTitleTextSize = values.GetFloat(Resource.Styleable.QuickActionsView_itemTitleTextSize, DefaultItemTitleTextSizeInSp);
                _itemTitleColor = values.GetColor(Resource.Styleable.QuickActionsView_itemTitleColor, Resource.Color.GrayTextColor);
                _itemSelectedTitleColor = values.GetColor(Resource.Styleable.QuickActionsView_itemSelectedTitleColor, 0);
                _itemSubTitleColor = values.GetColor(Resource.Styleable.QuickActionsView_itemSubTitleColor, Resource.Color.GrayTextColor);
                _itemSelectedSubTitleColor = values.GetColor(Resource.Styleable.QuickActionsView_itemSelectedSubTitleColor, 0);
                _itemIconResource = values.GetResourceId(Resource.Styleable.QuickActionsView_itemIconResource, 0);
                _itemSelectedIconResource = values.GetResourceId(Resource.Styleable.QuickActionsView_itemSelectedIconResource, 0);
            }
            finally
            {
                values.Recycle();
            }
        }


        void InitView()
        {
            SetBackgroundColor(Color.White);
            var layoutInflater = LayoutInflater.FromContext(Context);
            var layoutView = layoutInflater.Inflate(Resource.Layout.QuickActionsView, null);
            layoutView.LayoutParameters = new LayoutParams(ViewGroup.LayoutParams.MatchParent, ViewGroup.LayoutParams.WrapContent);

            _categoriesContainer = layoutView.FindViewById<LinearLayout>(Resource.Id.QuickActionsView_CategoriesContainer);
            _itemsRecyclerView = layoutView.FindViewById<MvxRecyclerView>(Resource.Id.QuickActionsView_ItemsRecyclerView);
            _layoutManager = new LinearLayoutManager(Context, LinearLayoutManager.Horizontal, reverseLayout: false);

            IMvxAndroidBindingContext bindingContext = null;

            if (Context is IMvxAndroidView mvxAndroidView && mvxAndroidView.BindingContext is IMvxAndroidBindingContext)
                bindingContext = (IMvxAndroidBindingContext)mvxAndroidView.BindingContext;
            else if (Context is IMvxFragmentView mvxFragmentView && mvxFragmentView.BindingContext is IMvxAndroidBindingContext)
                bindingContext = (IMvxAndroidBindingContext)mvxFragmentView.BindingContext;

            if (bindingContext == null)
            {
                Visibility = ViewStates.Gone;
                return;
            }

            _itemsRecyclerView.SetLayoutManager(_layoutManager);
            _itemsRecyclerView.HorizontalScrollBarEnabled = false;

            InitializeMode(bindingContext);

            AddView(layoutView);
        }


        TextView GetCategoryTextView(string category)
        {
            var textView = new TextView(Context)
            {
                LayoutParameters = new LayoutParams(ViewGroup.LayoutParams.WrapContent, ViewGroup.LayoutParams.WrapContent),
                Text = category,
                Tag = category,
                Ellipsize = TruncateAt.End
            };

            textView.SetTextColor(_categoryTitleColor);
            textView.SetTextSize(ComplexUnitType.Sp, CategoryTextSizeInSp);
            textView.SetTypeface(_categoryTitleTypeface, TypefaceStyle.Normal);
            textView.SetSingleLine();
            textView.SetOnClickListener(new OnClickListener(() => SelectCategory(_categoriesContainer.IndexOfChild(textView))));

            return textView;
        }


        void InitializeItems()
        {
            _categoriesContainer.RemoveAllViews();
            _itemsRecyclerView.ItemsSource = null;
            _selectedCategoryIndex = -1;

            if (Items == null || Items.CountSafely() <= 0)
            {
                Visibility = ViewStates.Gone;
                return;
            }

            Visibility = ViewStates.Visible;

            var itemsCount = Items.CountSafely();
            var totalItems = new List<IQuickActionSectionItem>();
            for (int i = 0; i < itemsCount; i++)
            {
                totalItems.AddRange(Items.ElementAt(i));

                if (_simpleMode)
                    continue;

                if (i == 0)
                {
                    _categoriesContainer.AddView(GetCategoryTextView(Items.ElementAt(i).GroupId));
                    continue;
                }

                var groupLayoutParams = new LayoutParams(ViewGroup.LayoutParams.WrapContent, ViewGroup.LayoutParams.WrapContent)
                {
                    LeftMargin = CategoryTitleLeftMarginInDp.DpToPx(Context)
                };
                _categoriesContainer.AddView(GetCategoryTextView(Items.ElementAt(i).GroupId), groupLayoutParams);
            }

            _itemsRecyclerView.ItemsSource = totalItems;

            if (!_simpleMode)
                SelectCategory(0);
        }


        void InitializeMode(IMvxAndroidBindingContext bindingContext)
        {
            if (_simpleMode)
            {
                _categoriesContainer.Visibility = ViewStates.Gone;
                _adapter = new HeaderFooterMvxRecyclerAdapter<QuickActionsSimpleItemView>(bindingContext, AdjustQuickActionsSimpleItemView);
                _adapter.ItemTemplateId = Resource.Layout.QuickActionsSimpleItemView;
                _itemsRecyclerView.Adapter = _adapter;
                return;
            }

            _categoriesContainer.Visibility = ViewStates.Visible;
            _adapter = new HeaderFooterMvxRecyclerAdapter<QuickActionsItemView>(bindingContext, AdjustQuickActionsItemView);
            _adapter.ItemTemplateId = Resource.Layout.QuickActionsItemView;
            _itemsRecyclerView.Adapter = _adapter;

            var scrollListener = new RecyclerViewOnScrollListener()
            {
                OnScrolledAction = ScrollListener_OnScrolled,
                OnScrollStateChangedAction = ScrollListener_OnScrollStateChanged
            };
            _itemsRecyclerView.AddOnScrollListener(scrollListener);

            _categorySelectedTitleTypeface = Typeface.Create(SelectedCategoryFontFamily, TypefaceStyle.Normal);
            _categoryTitleTypeface = Typeface.Create(Typeface.SansSerif, TypefaceStyle.Normal);
        }


        QuickActionsItemView AdjustQuickActionsItemView(QuickActionsItemView itemView)
        {
            itemView.ApplyStyles(
                   backgroundResource: _itemBackgroundResource,
                   selectedBackgroundResource: _itemSelectedBackgroundResource,
                   titleTextSize: _itemTitleTextSize,
                   titleColor: _itemTitleColor,
                   selectedTitleColor: _itemSelectedTitleColor,
                   subTitleColor: _itemSubTitleColor,
                   selectedSubTitleColor: _itemSelectedSubTitleColor,
                   iconResource: _itemIconResource,
                   selectedIconResource: _itemSelectedIconResource);

            return itemView;
        }


        QuickActionsSimpleItemView AdjustQuickActionsSimpleItemView(QuickActionsSimpleItemView itemView)
        {
            itemView.ApplyStyles(
                   backgroundResource: _itemBackgroundResource,
                   selectedBackgroundResource: _itemSelectedBackgroundResource,
                   titleTextSize: _itemTitleTextSize,
                   titleColor: _itemTitleColor,
                   selectedTitleColor: _itemSelectedTitleColor,
                   subTitleColor: _itemSubTitleColor,
                   selectedSubTitleColor: _itemSelectedSubTitleColor,
                   iconResource: _itemIconResource,
                   selectedIconResource: _itemSelectedIconResource);

            return itemView;
        }


        void SelectCategory(int categoryIndex, bool tryScroll = true)
        {
            if (categoryIndex < 0 || categoryIndex >= _categoriesContainer.ChildCount)
                return;

            HighLightSelectedCategory(categoryIndex);

            if (tryScroll)
                SmoothScrollToSelectedCategory();
        }


        void HighLightSelectedCategory(int categoryIndex)
        {
            if (_selectedCategoryIndex == categoryIndex)
                return;

            if (_selectedCategoryIndex >= 0)
            {
                var unselectedTextView = GetSelectedCategoryTextView();
                unselectedTextView.SetTypeface(_categoryTitleTypeface, TypefaceStyle.Normal);
                unselectedTextView.SetTextColor(_categoryTitleColor);
            }

            _selectedCategoryIndex = categoryIndex;

            var selectedTextView = GetSelectedCategoryTextView();
            selectedTextView.SetTypeface(_categorySelectedTitleTypeface, TypefaceStyle.Normal);
            selectedTextView.SetTextColor(_categorySelectedTitleColor);
        }


        void SmoothScrollToSelectedCategory()
        {
            if (_adapter.ItemsSource.IsNullOrEmpty())
                return;

            var items = (List<IQuickActionSectionItem>)_adapter.ItemsSource;
            var firstCategoryItem = items?.FirstOrDefault(i => string.Equals(i.RootGroupId, GetSelectedCategoryTextView().Text, StringComparison.OrdinalIgnoreCase));

            SmoothScrollToSelectedItem(items, firstCategoryItem);
        }


        void SmoothScrollToSelectedItem(List<IQuickActionSectionItem> items, IQuickActionSectionItem item)
        {
            if (item == null)
                return;

            int itemPosition = items.IndexOf(item);
            int firstVisiblePosition = _layoutManager.FindFirstCompletelyVisibleItemPosition();
            int lastVisiblePosition = _layoutManager.FindLastCompletelyVisibleItemPosition();

            if (firstVisiblePosition <= itemPosition && itemPosition <= lastVisiblePosition)
                return;

            var smoothScroller = new ExtendedLinearSmoothScroller(Context)
            {
                TargetPosition = itemPosition
            };

            _isScrollingByUser = false;
            _layoutManager.StartSmoothScroll(smoothScroller);
        }


        TextView GetSelectedCategoryTextView() => (TextView)_categoriesContainer.GetChildAt(_selectedCategoryIndex);


        void ScrollListener_OnScrolled(RecyclerView recyclerView, int dx, int dy)
        {
            if (!_isScrollingByUser)
                return;

            var firstVisiblePosition = _layoutManager.FindFirstVisibleItemPosition();

            if (firstVisiblePosition == RecyclerView.NoPosition)
                return;

            var firstVisibleView = _layoutManager.FindViewByPosition(firstVisiblePosition);

            if (firstVisibleView.Right - NeighboringItemsSideMarginInDp.DpToPx(Context) <= 0)
                firstVisiblePosition++;

            var firstVisibleItem = (IQuickActionSectionItem)_adapter.GetItem(firstVisiblePosition);
            var selectedCategory = GetSelectedCategoryTextView().Text;

            if (firstVisibleItem == null || string.Equals(firstVisibleItem.RootGroupId, selectedCategory, StringComparison.OrdinalIgnoreCase))
                return;

            var newCategoryIndex = _categoriesContainer.IndexOfChild(_categoriesContainer.FindViewWithTag(firstVisibleItem.RootGroupId));

            SelectCategory(newCategoryIndex, tryScroll: false);
        }


        void ScrollListener_OnScrollStateChanged(RecyclerView recyclerView, int newState)
        {
            if (newState == RecyclerView.ScrollStateDragging)
                _isScrollingByUser = true;
        }


        void SelectItem()
        {
            var items = (List<IQuickActionSectionItem>)_adapter.ItemsSource;
            if (items.IsNullOrEmpty())
                return;

            var currentSelectedItem = items.FirstOrDefault(i => i.IsSelected);
            if (currentSelectedItem != null)
                currentSelectedItem.IsSelected = false;

            var selectedItem = items.FirstOrDefault(i => i.Id == _selectedItem?.Id);
            if (selectedItem == null)
                return;

            selectedItem.IsSelected = true;
            SmoothScrollToSelectedItem(items, selectedItem);
        }


        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
        }
    }
}